SystemOrganization addCategory: #'BrpExtensions-Collections'!

!Object methodsFor: '*BrpExtensions' stamp: 'cmm 11/23/2010 21:03'!
, anObjectOrCollection
	^ self asCollection, anObjectOrCollection asCollection! !

!Object methodsFor: '*BrpExtensions' stamp: 'brp 5/9/2005 15:58'!
asArray

	^ Array with: self

! !

!Object methodsFor: '*BrpExtensions' stamp: 'brp 5/9/2005 15:56'!
asCollection

	^ self asArray! !

!Object methodsFor: '*BrpExtensions' stamp: 'brp 12/22/2009 08:51'!
if: aCondition

	^ self if: aCondition else: nil
! !

!Object methodsFor: '*BrpExtensions' stamp: 'brp 12/22/2009 08:51'!
if: aCondition else: anObject

	^ aCondition value
		ifTrue: [ self ]
		ifFalse: [ anObject ]! !

!Dictionary methodsFor: '*brpExtensions' stamp: 'brp 9/26/2005 06:48'!
at: key ifAbsent: absentBlock ifPresent: presentBlock 
	"Answer the result of evaluating presentBlock with the value associated with the key or, 
	if key isn't found, answer the result of evaluating absentBlock."

	^ self at: key ifPresent: presentBlock ifAbsent: absentBlock ! !

!Dictionary methodsFor: '*brpExtensions' stamp: 'brp 12/4/2009 14:13'!
at: key ifPresent: presentBlock ifAbsentPut: absentBlock 
	"Answer the result of evaluating presentBlock with the value associated with the key or, 
	if key isn't found, store the result of evaluating absentBlock."

	^ (array at: (self findElementOrNil: key))
		ifNil: [ self at: key put: absentBlock value ]
		ifNotNilDo: [ :assoc | presentBlock value: assoc value ]! !

!ByteArray methodsFor: '*BrpExtensions' stamp: 'cmm 11/22/2012 13:07'!
, otherCollection
	"Overidden because ByteArrays should not support object concatenation."
	^ self copyReplaceFrom: self size + 1
		  to: self size
		  with: otherCollection! !

ClassTestCase subclass: #MedleyTest
	instanceVariableNames: 'collections medley'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BrpExtensions-Collections'!

!MedleyTest class methodsFor: 'as yet unclassified' stamp: 'brp 5/4/2005 14:03'!
mustTestCoverage

	^ true! !

!MedleyTest methodsFor: 'Coverage' stamp: 'brp 5/4/2005 14:03'!
classToBeTested
	
	^ Medley! !

!MedleyTest methodsFor: 'Coverage' stamp: 'cmm 4/7/2012 16:18'!
selectorsToBeIgnored

	^ super selectorsToBeIgnored 
		, #(#setCollections: #collections)	"private or trivial"
		, #(#, #size #do: #new:) "special"! !

!MedleyTest methodsFor: 'Running' stamp: 'brp 5/4/2005 14:06'!
setUp

	super setUp.
	medley := Medley new! !

!MedleyTest methodsFor: 'Tests' stamp: 'brp 5/4/2005 14:38'!
testAddAndRemove

	| size |
	size := medley size.
	medley add: 1.
	self assert: (medley size = (size + 1)).
	medley remove: 1 ifAbsent: [].
	self assert: (medley size = size).

	medley add: 1; add: #(2 3); add: (4 to: 7); add: 8.
	self assert: (medley size = (size + 8)).

	medley remove: #(2 3).
	self assert: (medley size = (size + 8 - 2)).

	self should: [ medley remove: 0 ] raise: Error.

	medley add: #(11 12); add: (13 to: 16).
	size := medley size.
	medley remove: 12; remove: 15.
	self 
		assert: (medley size = (size - 2));
		deny: (medley includes: 12);
		deny: (medley includes: 15).

	
! !

!MedleyTest methodsFor: 'Tests' stamp: 'brp 5/4/2005 14:26'!
testConcatentation

	| concat appendix |
	medley add: 1; add: #(2 3); add: (4 to: 7); add: 8.
	appendix := #(9 10 11).
	concat := medley , appendix.

	self 
		assert: (concat size = (medley size + appendix size)).
! !

!MedleyTest methodsFor: 'Tests' stamp: 'brp 5/4/2005 14:23'!
testCopy

	| duplicate |
	medley add: 1; add: #(2 3); add: (4 to: 7); add: 8.
	duplicate := medley copy.

	self 
		assert: (medley size = duplicate size);
		assert: (medley collections = duplicate collections);
		deny: (medley collections == duplicate collections).! !

!MedleyTest methodsFor: 'Tests' stamp: 'brp 5/4/2005 14:20'!
testDo

	| elements |
	elements := OrderedCollection new.
	medley add: 1; add: #(2 3); add: (4 to: 7); add: 8.
	medley do: [ :x | elements add: x ].

	self assert: (elements = ((1 to: 8) asOrderedCollection)).
! !

!MedleyTest methodsFor: 'Tests' stamp: 'brp 5/4/2005 14:07'!
testEmpty

	self 
		assert: (medley size = 0);
		assert: (medley isEmpty).! !

!WordArray methodsFor: '*brpextensions' stamp: 'cmm 11/22/2012 13:08'!
, otherCollection 
	"Overidden to preserve standard concatentation for WordArray's."
	^ self copyReplaceFrom: self size + 1
		  to: self size
		  with: otherCollection! !

!Text methodsFor: '*BrpExtensions' stamp: 'cmm 11/22/2012 13:07'!
, otherCollection 
	"Overidden because Texts should not support object concatenation."
	^ self copyReplaceFrom: self size + 1
		  to: self size
		  with: otherCollection! !

!Text methodsFor: '*BrpExtensions' stamp: 'cmm 10/19/2012 17:41'!
asCollection
	"Texts like to pretend they're byte objects."
	^ Array with: self! !

!SimplifiedChineseEnvironment class methodsFor: '*BrpExtensions' stamp: 'brp 5/18/2005 15:10'!
leadingChar

	^ 0
! !

!Collection methodsFor: '*BrpExtensions' stamp: 'cmm 11/22/2012 13:04'!
asCollection
	"Answer a pointer collection with the receiver as the first-element."
	^ self class isPointers
		ifTrue: [ super asCollection ]
		ifFalse: [ Array with: self ]! !

Collection subclass: #Medley
	instanceVariableNames: 'collections'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'BrpExtensions-Collections'!

!Medley commentStamp: '<historical>' prior: 0!
A Medley is a Collection which contains individual objects or other collections.!

!Medley class methodsFor: 'instance creation' stamp: 'brp 1/16/2009 09:09'!
new: anInteger

	^ self new! !

!Medley methodsFor: 'copying' stamp: 'brp 3/29/2005 20:41'!
, aCollection

	^ self copyWith: aCollection! !

!Medley methodsFor: 'adding' stamp: 'brp 3/29/2005 20:17'!
add: aCollection
	"Include aCollection as one of the receiver's collections"

	^ self collections add: aCollection! !

!Medley methodsFor: 'private' stamp: 'cmm 4/7/2012 17:45'!
collections
	^ collections ifNil: [ collections := self collectionsClass new ]! !

!Medley methodsFor: 'private' stamp: 'cmm 4/7/2012 17:44'!
collectionsClass
	"Allow subclasses to override."
	^ OrderedCollection! !

!Medley methodsFor: 'copying' stamp: 'cmm 4/5/2012 19:51'!
copy
	"Make a shallow copy of the collections"
	^ self clone
		 setCollections:
			(self collections collect:
				[ : each | each isCollection
					ifTrue: [ each copy ]
					ifFalse: [ each ] ]) ;
		 yourself! !

!Medley methodsFor: 'private' stamp: 'brp 3/29/2005 20:37'!
do: aBlock 
	"Evaluate aBlock with each of the receiver's elements as the argument."

	self collections do: [ :each | 
		each isCollection ifTrue: [ each do: aBlock ] ifFalse: [ each in: aBlock ] ]! !

!Medley methodsFor: 'removing' stamp: 'brp 5/4/2005 14:35'!
remove: oldObject ifAbsent: anExceptionBlock 

	self collections remove: oldObject ifAbsent: [
		(self includes: oldObject)
			ifTrue: 
				[ | holder |
					holder := (self collections select: [ :c | c isCollection ])
							detect: [ :c | c includes: oldObject ].
					self 
						remove: holder;
						add: (holder copyWithout: oldObject) ]
			ifFalse: 
				[ anExceptionBlock value ]
	]! !

!Medley methodsFor: 'private' stamp: 'brp 3/29/2005 20:30'!
setCollections: anOrderedCollection

	collections := anOrderedCollection! !

!Medley methodsFor: 'accessing' stamp: 'brp 3/29/2005 20:38'!
size
	"Answer how many elements the receiver contains."

	| tally |
	tally := 0.
	self collections do: [ :each | tally := tally + (each isCollection ifTrue: [each size] ifFalse: [1]) ].
	^ tally! !
